const std = @import("std");
const yazap = @import("yazap");
const root = @import("root.zig");

const allocator = std.heap.page_allocator;
const log = std.log;
const App = yazap.App;
const Arg = yazap.Arg;

pub fn main() !u8 {
    var app = App.init(allocator, "mpar", "Mozaic pack archiver");
    defer app.deinit();

    var mpar = app.rootCommand();
    mpar.setProperty(.help_on_empty_args);

    var list_cmd = app.createCommand("l", "List files from a pack");
    list_cmd.setProperty(.help_on_empty_args);
    list_cmd.setProperty(.positional_arg_required);

    try list_cmd.addArgs(&[_]Arg{
        Arg.singleValueOptionWithValidValues("format", 'f', "Output format", &[_][]const u8{ "none", "json" }),
        Arg.positional("filename", "Path to .pak file", null),
    });

    try mpar.addSubcommand(list_cmd);

    var extract_cmd = app.createCommand("x", "Extract files from a pack");
    extract_cmd.setProperty(.help_on_empty_args);
    extract_cmd.setProperty(.positional_arg_required);

    try extract_cmd.addArgs(&[_]Arg{
        Arg.singleValueOption("output", 'o', "Output directory"),
        Arg.booleanOption("quiet", 'q', "Be quiet"),
        Arg.positional("filename", "Path to .pak file", null),
    });

    try mpar.addSubcommand(extract_cmd);

    const matches = try app.parseProcess();

    if (matches.subcommandMatches("x")) |submatches| {
        return root.extract(submatches);
    }

    if (matches.subcommandMatches("l")) |submatches| {
        return root.list(submatches);
    }

    return 2;
}
