const std = @import("std");
const yazap = @import("yazap");
const Pack = @import("pack.zig");
const allocator = std.heap.page_allocator;

pub fn list(matches: yazap.ArgMatches) !u8 {
    const name = matches.getSingleValue("filename").?;
    const pack = Pack.open(allocator, name) catch |err| switch (err) {
        error.InvalidPack => {
            std.log.err("invalid pack", .{});
            return 1;
        },
        else => {
            std.log.err("read error", .{});
            return 1;
        },
    };
    defer pack.close();
    const stdout = std.io.getStdOut().writer();

    if (matches.getSingleValue("format")) |format| {
        if (std.mem.eql(u8, format, "json")) {
            try stdout.print("{{\n  \"description\": \"", .{});
            try std.zig.stringEscape(pack.description, "", .{}, stdout);
            try stdout.print("\",\n  \"entries\": [\n", .{});
            var i: usize = 0;
            for (pack.entries) |entry| {
                i += 1;
                try stdout.print("    {{\n      \"name\": \"", .{});
                try std.zig.stringEscape(entry.oname, "", .{}, stdout);
                try stdout.print("\",\n      \"value\": {d}\n", .{entry.value});
                if (i == pack.entries.len) {
                    try stdout.print("    }}\n", .{});
                } else {
                    try stdout.print("    }},\n", .{});
                }
            }
            try stdout.print("  ]\n}}\n", .{});

            return 0;
        }
    }

    if (!std.mem.eql(u8, pack.description, ""))
        try stdout.print("{s}\n\n", .{pack.description});
    try stdout.print("      size val name\n", .{});
    try stdout.print("{s:-<48}\n", .{""});
    var size: u32 = 0;
    var n: u32 = 0;
    for (pack.entries) |entry| {
        size += entry.size;
        n += 1;
        try std.io.getStdOut().writer().print("{d: >10} {d: >3} {s}\n", .{ entry.size, entry.value, entry.name });
    }
    try stdout.print("{s:-<48}\n", .{""});
    try stdout.print("{d: >10} bytes in {} files\n", .{ size, n });
    return 0;
}

pub fn extract(matches: yazap.ArgMatches) !u8 {
    const name = matches.getSingleValue("filename").?;
    const pack = Pack.open(allocator, name) catch |err| switch (err) {
        error.InvalidPack => {
            std.log.err("invalid pack", .{});
            return 1;
        },
        else => {
            std.log.err("read error", .{});
            return 1;
        },
    };
    defer pack.close();

    var currentDir = std.fs.cwd();
    if (matches.getSingleValue("output")) |path| {
        try std.fs.cwd().makePath(path);
        currentDir = try std.fs.cwd().openDir(path, .{});
    }

    for (pack.entries) |entry| {
        if (std.fs.path.dirname(entry.name)) |dir| {
            try currentDir.makePath(dir);
        }
        const f = try currentDir.createFile(entry.name, .{});
        defer f.close();

        _ = try pack.copy(entry, f, 0);

        if (!matches.containsArg("quiet")) {
            try std.io.getStdOut().writer().print("{s}\n", .{entry.name});
        }
    }

    return 0;
}
